@@ -149,9 +149,6 @@ class DataSmart:
 
         if not var in self.dict:
             self._makeShadowCopy(var)
-        if self.getVarFlag(var, 'matchesenv'):
-            self.delVarFlag(var, 'matchesenv')
-            self.setVarFlag(var, 'export', 1)
 
         # more cookies for the cookie monster
         if '_' in var:
@@ -170,6 +167,28 @@ class DataSmart:
             return self.expand(value,var)
         return value
 
+    def renameVar(self, key, newkey):
+        """
+        Rename the variable key to newkey 
+        """
+        val = self.getVar(key, 0)
+        if val is None:
+            return
+
+        self.setVar(newkey, val)
+
+        for i in ('_append', '_prepend'):
+            dest = self.getVarFlag(newkey, i) or []
+            src = self.getVarFlag(key, i) or []
+            dest.extend(src)
+            self.setVarFlag(newkey, i, dest)
+            
+            if self._special_values.has_key(i) and key in self._special_values[i]:
+                self._special_values[i].remove(key)
+                self._special_values[i].add(newkey)
+
+        self.delVar(key)
+
     def delVar(self,var):
         self.expand_cache = {}
         self.dict[var] = {}
@@ -210,10 +229,10 @@ class DataSmart:
         flags = {}
 
         if local_var:
-            for i in self.dict[var].keys():
+            for i in local_var.keys():
                 if i == "content":
                     continue
-                flags[i] = self.dict[var][i]
+                flags[i] = local_var[i]
 
         if len(flags) == 0:
             return None
