@@ -49,6 +49,9 @@ class ParameterError(Exception):
 class MD5SumError(Exception):
     """Exception raised when a MD5SUM of a file does not match the expected one"""
 
+class InvalidSRCREV(Exception):
+    """Exception raised when an invalid SRCREV is encountered"""
+
 def uri_replace(uri, uri_find, uri_replace, d):
 #   bb.msg.note(1, bb.msg.domain.Fetcher, "uri_replace: operating on %s" % uri)
     if not uri or not uri_find or not uri_replace:
@@ -139,13 +142,21 @@ def go(d):
             if not m.forcefetch(u, ud, d) and os.path.exists(ud.md5):
                 # File already present along with md5 stamp file
                 # Touch md5 file to show activity
-                os.utime(ud.md5, None)
+                try:
+                    os.utime(ud.md5, None)
+                except:
+                    # Errors aren't fatal here
+                    pass
                 continue
             lf = bb.utils.lockfile(ud.lockfile)
             if not m.forcefetch(u, ud, d) and os.path.exists(ud.md5):
                 # If someone else fetched this before we got the lock, 
                 # notice and don't try again
-                os.utime(ud.md5, None)
+                try:
+                    os.utime(ud.md5, None)
+                except:
+                    # Errors aren't fatal here
+                    pass
                 bb.utils.unlockfile(lf)
                 continue
         m.go(u, ud, d)
@@ -154,6 +165,22 @@ def go(d):
                 Fetch.write_md5sum(u, ud, d)
             bb.utils.unlockfile(lf)
 
+
+def checkstatus(d):
+    """
+    Check all urls exist upstream
+    init must have previously been called
+    """
+    urldata = init([], d, True)
+
+    for u in urldata:
+        ud = urldata[u]
+        m = ud.method
+        bb.msg.note(1, bb.msg.domain.Fetcher, "Testing URL %s" % u)
+        ret = m.checkstatus(u, ud, d)
+        if not ret:
+            bb.msg.fatal(bb.msg.domain.Fetcher, "URL %s doesn't work" % u)
+
 def localpaths(d):
     """
     Return a list of the local filenames, assuming successful fetch
@@ -205,6 +232,8 @@ def get_srcrev(d):
         bb.msg.error(bb.msg.domain.Fetcher, "SRCREV was used yet no valid SCM was found in SRC_URI")
         raise ParameterError
 
+    bb.data.setVar('__BB_DONT_CACHE','1', d)
+
     if len(scms) == 1:
         return urldata[scms[0]].method.sortable_revision(scms[0], urldata[scms[0]], d)
 
@@ -240,13 +269,22 @@ def runfetchcmd(cmd, d, quiet = False):
     Raise an error if interrupted or cmd fails
     Optionally echo command output to stdout
     """
-    bb.msg.debug(1, bb.msg.domain.Fetcher, "Running %s" % cmd)
 
     # Need to export PATH as binary could be in metadata paths
     # rather than host provided
-    pathcmd = 'export PATH=%s; %s' % (data.expand('${PATH}', d), cmd)
+    # Also include some other variables.
+    # FIXME: Should really include all export varaiables?
+    exportvars = ['PATH', 'GIT_PROXY_HOST', 'GIT_PROXY_PORT', 'GIT_CONFIG', 'http_proxy', 'ftp_proxy', 'SSH_AUTH_SOCK', 'SSH_AGENT_PID']
 
-    stdout_handle = os.popen(pathcmd, "r")
+    for var in exportvars:
+        val = data.getVar(var, d, True)
+        if val:
+            cmd = 'export ' + var + '=%s; %s' % (val, cmd)
+
+    bb.msg.debug(1, bb.msg.domain.Fetcher, "Running %s" % cmd)
+
+    # redirect stderr to stdout
+    stdout_handle = os.popen(cmd + " 2>&1", "r")
     output = ""
 
     while 1:
@@ -262,9 +300,9 @@ def runfetchcmd(cmd, d, quiet = False):
     exitstatus = status & 0xff
 
     if signal:
-        raise FetchError("Fetch command %s failed with signal %s, output:\n%s" % (pathcmd, signal, output))
+        raise FetchError("Fetch command %s failed with signal %s, output:\n%s" % (cmd, signal, output))
     elif status != 0:
-        raise FetchError("Fetch command %s failed with exit code %s, output:\n%s" % (pathcmd, status, output))
+        raise FetchError("Fetch command %s failed with exit code %s, output:\n%s" % (cmd, status, output))
 
     return output
 
@@ -277,6 +315,10 @@ class FetchData(object):
         (self.type, self.host, self.path, self.user, self.pswd, self.parm) = bb.decodeurl(data.expand(url, d))
         self.date = Fetch.getSRCDate(self, d)
         self.url = url
+        if not self.user and "user" in self.parm:
+            self.user = self.parm["user"]
+        if not self.pswd and "pswd" in self.parm:
+            self.pswd = self.parm["pswd"]
         self.setup = False
         for m in methods:
             if m.supports(url, self, d):
@@ -347,6 +389,14 @@ class Fetch(object):
         """
         raise NoMethodError("Missing implementation for url")
 
+    def checkstatus(self, url, urldata, d):
+        """
+        Check the status of a URL
+        Assumes localpath was called first
+        """
+        bb.msg.note(1, bb.msg.domain.Fetcher, "URL %s could not be checked for status since no method exists." % url)
+        return True
+
     def getSRCDate(urldata, d):
         """
         Return the SRC Date for the component
@@ -384,6 +434,8 @@ class Fetch(object):
             rev = data.getVar("SRCREV_pn-" + pn + "_" + ud.parm['name'], d, 1)
         if not rev:
             rev = data.getVar("SRCREV", d, 1)
+        if rev == "INVALID":
+            raise InvalidSRCREV("Please set SRCREV to a valid value")
         if not rev:
             return False
         if rev is "SRCREVINACTION":
@@ -417,6 +469,12 @@ class Fetch(object):
             uri = stash + tarfn
             bb.msg.note(1, bb.msg.domain.Fetcher, "fetch " + uri)
             fetchcmd = fetchcmd.replace("${URI}", uri)
+            httpproxy = data.getVar("http_proxy", d, True)
+            ftpproxy = data.getVar("ftp_proxy", d, True)
+            if httpproxy:
+                fetchcmd = "http_proxy=" + httpproxy + " " + fetchcmd
+            if ftpproxy:
+                fetchcmd = "ftp_proxy=" + ftpproxy + " " + fetchcmd
             ret = os.system(fetchcmd)
             if ret == 0:
                 bb.msg.note(1, bb.msg.domain.Fetcher, "Fetched %s from tarball stash, skipping checkout" % tarfn)
@@ -438,14 +496,7 @@ class Fetch(object):
     verify_md5sum = staticmethod(verify_md5sum)
 
     def write_md5sum(url, ud, d):
-        if bb.which(data.getVar('PATH', d), 'md5sum'):
-            try:
-                md5pipe = os.popen('md5sum ' + ud.localpath)
-                md5data = (md5pipe.readline().split() or [ "" ])[0]
-                md5pipe.close()
-            except OSError:
-                md5data = ""
-
+        md5data = bb.utils.md5_file(ud.localpath)
         # verify the md5sum
         if not Fetch.verify_md5sum(ud, md5data):
             raise MD5SumError(url)
@@ -463,7 +514,7 @@ class Fetch(object):
             raise ParameterError
 
         pd = persist_data.PersistData(d)
-        key = self._revision_key(url, ud, d)
+        key = self.generate_revision_key(url, ud, d)
         rev = pd.getValue("BB_URI_HEADREVS", key)
         if rev != None:
             return str(rev)
@@ -476,11 +527,18 @@ class Fetch(object):
         """
         
         """
-        if hasattr(self, "_sortable_revision"):
+        has_want_sortable = hasattr(self, "_want_sortable_revision")
+        has_sortable = hasattr(self, "_sortable_revision")
+
+        if not has_want_sortable and has_sortable:
             return self._sortable_revision(url, ud, d)
+        elif has_want_sortable and self._want_sortable_revision(url, ud, d) and has_sortable:
+            return self._sortable_revision(url, ud, d)
+        
 
         pd = persist_data.PersistData(d)
-        key = self._revision_key(url, ud, d)
+        key = self.generate_revision_key(url, ud, d)
+
         latest_rev = self._build_revision(url, ud, d)
         last_rev = pd.getValue("BB_URI_LOCALCOUNT", key + "_rev")
         count = pd.getValue("BB_URI_LOCALCOUNT", key + "_count")
@@ -498,6 +556,9 @@ class Fetch(object):
 
         return str(count + "+" + latest_rev)
 
+    def generate_revision_key(self, url, ud, d):
+        key = self._revision_key(url, ud, d)
+        return "%s-%s" % (key, bb.data.getVar("PN", d, True) or "")
 
 import cvs
 import git
@@ -509,6 +570,7 @@ import ssh
 import perforce
 import bzr
 import hg
+import osc
 
 methods.append(local.Local())
 methods.append(wget.Wget())
@@ -520,3 +582,4 @@ methods.append(ssh.SSH())
 methods.append(perforce.Perforce())
 methods.append(bzr.Bzr())
 methods.append(hg.Hg())
+methods.append(osc.Osc())
