@@ -213,34 +213,6 @@ def _filterProviders(providers, item, cfgData, dataCache):
         eligible.remove(fn)
         eligible = [fn] + eligible
 
-    # look to see if one of them is already staged, or marked as preferred.
-    # if so, bump it to the head of the queue
-    for p in providers:
-        pn = dataCache.pkg_fn[p]
-        pe, pv, pr = dataCache.pkg_pepvpr[p]
-
-        stamp = '%s.do_populate_staging' % dataCache.stamp[p]
-        if os.path.exists(stamp):
-            (newvers, fn) = preferred_versions[pn]
-            if not fn in eligible:
-                # package was made ineligible by already-failed check
-                continue
-            oldver = "%s-%s" % (pv, pr)
-            if pe > 0:
-                oldver = "%s:%s" % (pe, oldver)
-            newver = "%s-%s" % (newvers[1], newvers[2])
-            if newvers[0] > 0:
-                newver = "%s:%s" % (newvers[0], newver)
-            if (newver != oldver):
-                extra_chat = "%s (%s) already staged but upgrading to %s to satisfy %s" % (pn, oldver, newver, item)
-            else:
-                extra_chat = "Selecting already-staged %s (%s) to satisfy %s" % (pn, oldver, item)
-
-            bb.msg.note(2, bb.msg.domain.Provider, "%s" % extra_chat)
-            eligible.remove(fn)
-            eligible = [fn] + eligible
-            break
-
     return eligible
 
 
@@ -283,13 +255,17 @@ def filterProvidersRunTime(providers, item, cfgData, dataCache):
 
     # Should use dataCache.preferred here?
     preferred = []
+    preferred_vars = []
     for p in eligible:
         pn = dataCache.pkg_fn[p]
         provides = dataCache.pn_provides[pn]
         for provide in provides:
+            bb.msg.note(2, bb.msg.domain.Provider, "checking PREFERRED_PROVIDER_%s" % (provide))
             prefervar = bb.data.getVar('PREFERRED_PROVIDER_%s' % provide, cfgData, 1)
             if prefervar == pn:
-                bb.msg.note(2, bb.msg.domain.Provider, "selecting %s to satisfy runtime %s due to PREFERRED_PROVIDERS" % (pn, item))
+                var = "PREFERRED_PROVIDER_%s = %s" % (provide, prefervar)
+                bb.msg.note(2, bb.msg.domain.Provider, "selecting %s to satisfy runtime %s due to %s" % (pn, item, var))
+                preferred_vars.append(var)
                 eligible.remove(p)
                 eligible = [p] + eligible
                 preferred.append(p)
@@ -297,6 +273,9 @@ def filterProvidersRunTime(providers, item, cfgData, dataCache):
 
     numberPreferred = len(preferred)
 
+    if numberPreferred > 1:
+        bb.msg.error(bb.msg.domain.Provider, "Conflicting PREFERRED_PROVIDER entries were found which resulted in an attempt to select multiple providers (%s) for runtime dependecy %s\nThe entries resulting in this conflict were: %s" % (preferred, item, preferred_vars))
+
     bb.msg.debug(1, bb.msg.domain.Provider, "sorted providers for %s are: %s" % (item, eligible))
 
     return eligible, numberPreferred
@@ -318,7 +297,12 @@ def getRuntimeProviders(dataCache, rdepend):
 
     # Only search dynamic packages if we can't find anything in other variables
     for pattern in dataCache.packages_dynamic:
-        regexp = re.compile(pattern)
+        pattern = pattern.replace('+', "\+")
+        try:
+            regexp = re.compile(pattern)
+        except:
+            bb.msg.error(bb.msg.domain.Provider, "Error parsing re expression: %s" % pattern)
+            raise
         if regexp.match(rdepend):
             rproviders += dataCache.packages_dynamic[pattern]
 
