@@ -26,6 +26,7 @@ from bb import msg, data, event, mkdirhier, utils
 from sets import Set 
 import bb, os, sys
 import signal
+import stat
 
 class TaskFailure(Exception):
     """Exception raised when a task in a runqueue fails"""
@@ -45,11 +46,11 @@ class RunQueueStats:
     def taskFailed(self):
         self.failed = self.failed + 1
 
-    def taskCompleted(self):
-        self.completed = self.completed + 1
+    def taskCompleted(self, number = 1):
+        self.completed = self.completed + number
 
-    def taskSkipped(self):
-        self.skipped = self.skipped + 1
+    def taskSkipped(self, number = 1):
+        self.skipped = self.skipped + number
 
 class RunQueueScheduler:
     """
@@ -144,8 +145,12 @@ class RunQueue:
         self.taskData = taskData
         self.targets = targets
 
-        self.number_tasks = int(bb.data.getVar("BB_NUMBER_THREADS", cfgData) or 1)
-        self.multi_provider_whitelist = (bb.data.getVar("MULTI_PROVIDER_WHITELIST", cfgData) or "").split()
+        self.cfgdata = cfgData
+        self.number_tasks = int(bb.data.getVar("BB_NUMBER_THREADS", cfgData, 1) or 1)
+        self.multi_provider_whitelist = (bb.data.getVar("MULTI_PROVIDER_WHITELIST", cfgData, 1) or "").split()
+        self.scheduler = bb.data.getVar("BB_SCHEDULER", cfgData, 1) or "speed"
+        self.stamppolicy = bb.data.getVar("BB_STAMP_POLICY", cfgData, 1) or "perfile"
+        self.stampwhitelist = bb.data.getVar("BB_STAMP_WHITELIST", cfgData, 1) or ""
 
     def reset_runqueue(self):
 
@@ -159,6 +164,12 @@ class RunQueue:
         taskname = self.runq_task[task]
         return "%s, %s" % (fn, taskname)
 
+    def get_task_id(self, fnid, taskname):
+        for listid in range(len(self.runq_fnid)):
+            if self.runq_fnid[listid] == fnid and self.runq_task[listid] == taskname:
+                return listid
+        return None
+
     def circular_depchains_handler(self, tasks):
         """
         Some tasks aren't buildable, likely due to circular dependency issues.
@@ -313,6 +324,7 @@ class RunQueue:
 
         depends = []
         runq_build = []
+        recursive_tdepends = {}
 
         taskData = self.taskData
 
@@ -378,14 +390,49 @@ class RunQueue:
                 # e.g. do_sometask[depends] = "targetname:do_someothertask"
                 # (makes sure sometask runs after targetname's someothertask)
                 idepends = taskData.tasks_idepends[task]
-                for idepend in idepends:
-                    depid = int(idepend.split(":")[0])
+                for (depid, idependtask) in idepends:
                     if depid in taskData.build_targets:
                         # Won't be in build_targets if ASSUME_PROVIDED
                         depdata = taskData.build_targets[depid][0]
                         if depdata is not None:
                             dep = taskData.fn_index[depdata]
-                            depends.append(taskData.gettask_id(dep, idepend.split(":")[1]))
+                            depends.append(taskData.gettask_id(dep, idependtask))
+
+                # Create a list of recursive dependent tasks (from tdepends) and cache
+                def get_recursive_tdepends(task):
+                    if not task:
+                        return []
+                    if task in recursive_tdepends:
+                        return recursive_tdepends[task]
+
+                    fnid = taskData.tasks_fnid[task]
+                    taskids = taskData.gettask_ids(fnid)
+
+                    rectdepends = taskids
+                    nextdeps = taskids
+                    while len(nextdeps) != 0:
+                        newdeps = []
+                        for nextdep in nextdeps:
+                            for tdepend in taskData.tasks_tdepends[nextdep]:
+                                if tdepend not in rectdepends:
+                                    rectdepends.append(tdepend)
+                                    newdeps.append(tdepend)
+                        nextdeps = newdeps
+                    recursive_tdepends[task] = rectdepends
+                    return rectdepends
+
+                # Using the list of tdepends for this task create a list of 
+                # the recursive idepends we have
+                def get_recursive_idepends(task):
+                    if not task:
+                        return []
+                    rectdepends = get_recursive_tdepends(task)
+
+                    recidepends = []
+                    for tdepend in rectdepends:
+                        for idepend in taskData.tasks_idepends[tdepend]:
+                            recidepends.append(idepend)
+                    return recidepends
 
                 def add_recursive_build(depid, depfnid):
                     """
@@ -400,13 +447,11 @@ class RunQueue:
                         depdata = taskData.build_targets[depid][0]
                         if depdata is not None:
                             dep = taskData.fn_index[depdata]
-                            idepends = []
                             # Need to avoid creating new tasks here
                             taskid = taskData.gettask_id(dep, taskname, False)
                             if taskid is not None:
                                 depends.append(taskid)
                                 fnid = taskData.tasks_fnid[taskid]
-                                idepends = taskData.tasks_idepends[taskid]
                                 #print "Added %s (%s) due to %s" % (taskid, taskData.fn_index[fnid], taskData.fn_index[depfnid])
                             else:
                                 fnid = taskData.getfn_id(dep)
@@ -416,10 +461,9 @@ class RunQueue:
                             for nextdepid in taskData.rdepids[fnid]:
                                 if nextdepid not in rdep_seen:
                                     add_recursive_run(nextdepid, fnid)
-                            for idepend in idepends:
-                                nextdepid = int(idepend.split(":")[0])
-                                if nextdepid not in dep_seen:
-                                    add_recursive_build(nextdepid, fnid)
+                            for (idependid, idependtask) in get_recursive_idepends(taskid):
+                                if idependid not in dep_seen:
+                                    add_recursive_build(idependid, fnid)
 
                 def add_recursive_run(rdepid, depfnid):
                     """
@@ -434,13 +478,11 @@ class RunQueue:
                         depdata = taskData.run_targets[rdepid][0]
                         if depdata is not None:
                             dep = taskData.fn_index[depdata]
-                            idepends = []
                             # Need to avoid creating new tasks here
                             taskid = taskData.gettask_id(dep, taskname, False)
                             if taskid is not None:
                                 depends.append(taskid)
                                 fnid = taskData.tasks_fnid[taskid]
-                                idepends = taskData.tasks_idepends[taskid]
                                 #print "Added %s (%s) due to %s" % (taskid, taskData.fn_index[fnid], taskData.fn_index[depfnid])
                             else:
                                 fnid = taskData.getfn_id(dep)
@@ -450,10 +492,9 @@ class RunQueue:
                             for nextdepid in taskData.rdepids[fnid]:
                                 if nextdepid not in rdep_seen:
                                     add_recursive_run(nextdepid, fnid)
-                            for idepend in idepends:
-                                nextdepid = int(idepend.split(":")[0])
-                                if nextdepid not in dep_seen:
-                                    add_recursive_build(nextdepid, fnid)
+                            for (idependid, idependtask) in get_recursive_idepends(taskid):
+                                if idependid not in dep_seen:
+                                    add_recursive_build(idependid, fnid)
 
                 # Resolve recursive 'recrdeptask' dependencies 
                 #
@@ -468,9 +509,9 @@ class RunQueue:
                             add_recursive_build(depid, fnid)
                         for rdepid in taskData.rdepids[fnid]:
                             add_recursive_run(rdepid, fnid)
-                        for idepend in idepends:
-                            depid = int(idepend.split(":")[0])
-                            add_recursive_build(depid, fnid)
+                        deptaskid = taskData.gettask_id(fn, taskname, False)
+                        for (idependid, idependtask) in get_recursive_idepends(deptaskid):
+                            add_recursive_build(idependid, fnid)
 
                 # Rmove all self references
                 if task in depends:
@@ -512,6 +553,7 @@ class RunQueue:
             for depend in depends:
                 mark_active(depend, depth+1)
 
+        self.target_pairs = []
         for target in self.targets:
             targetid = taskData.getbuild_id(target[0])
 
@@ -522,10 +564,11 @@ class RunQueue:
                 continue
 
             fnid = taskData.build_targets[targetid][0]
+            fn = taskData.fn_index[fnid]
+            self.target_pairs.append((fn, target[1]))
 
             # Remove stamps for targets if force mode active
             if self.cooker.configuration.force:
-                fn = taskData.fn_index[fnid]
                 bb.msg.note(2, bb.msg.domain.RunQueue, "Remove stamp %s, %s" % (target[1], fn))
                 bb.build.del_stamp(target[1], self.dataCache, fn)
 
@@ -564,9 +607,9 @@ class RunQueue:
         # Check to make sure we still have tasks to run
         if len(self.runq_fnid) == 0:
             if not taskData.abort:
-                bb.msg.note(1, bb.msg.domain.RunQueue, "All buildable tasks have been run but the build is incomplete (--continue mode). Errors for the tasks that failed will have been printed above.")
-                return
-            bb.msg.fatal(bb.msg.domain.RunQueue, "No active tasks and not in --continue mode?! Please report this bug.")
+                bb.msg.fatal(bb.msg.domain.RunQueue, "All buildable tasks have been run but the build is incomplete (--continue mode). Errors for the tasks that failed will have been printed above.")
+            else:		
+                bb.msg.fatal(bb.msg.domain.RunQueue, "No active tasks and not in --continue mode?! Please report this bug.")
 
         bb.msg.note(2, bb.msg.domain.RunQueue, "Pruned %s inactive tasks, %s left" % (delcount, len(self.runq_fnid)))
 
@@ -608,10 +651,11 @@ class RunQueue:
         self.runq_weight = self.calculate_task_weights(endpoints)
 
         # Decide what order to execute the tasks in, pick a scheduler
-        # FIXME - Allow user selection
         #self.sched = RunQueueScheduler(self)
-        self.sched = RunQueueSchedulerSpeed(self)
-        #self.sched = RunQueueSchedulerCompletion(self)
+        if self.scheduler == "completion":
+            self.sched = RunQueueSchedulerCompletion(self)
+        else:
+            self.sched = RunQueueSchedulerSpeed(self)
 
         # Sanity Check - Check for multiple tasks building the same provider
         prov_list = {}
@@ -634,8 +678,156 @@ class RunQueue:
         #if error:
         #    bb.msg.fatal(bb.msg.domain.RunQueue, "Corrupted metadata configuration detected, aborting...")
 
+
+        # Create a whitelist usable by the stamp checks
+        stampfnwhitelist = []
+        for entry in self.stampwhitelist.split():
+            entryid = self.taskData.getbuild_id(entry)
+            if entryid not in self.taskData.build_targets:
+                continue
+            fnid = self.taskData.build_targets[entryid][0]
+            fn = self.taskData.fn_index[fnid]
+            stampfnwhitelist.append(fn)
+        self.stampfnwhitelist = stampfnwhitelist
+
         #self.dump_data(taskData)
 
+    def check_stamps(self):
+        unchecked = {}
+        current = []
+        notcurrent = []
+        buildable = []
+
+        if self.stamppolicy == "perfile":
+            fulldeptree = False
+        else:
+            fulldeptree = True
+            stampwhitelist = []
+            if self.stamppolicy == "whitelist":
+                stampwhitelist = self.self.stampfnwhitelist
+
+        for task in range(len(self.runq_fnid)):
+            unchecked[task] = ""
+            if len(self.runq_depends[task]) == 0:
+                buildable.append(task)
+
+        def check_buildable(self, task, buildable):
+             for revdep in self.runq_revdeps[task]:
+                alldeps = 1
+                for dep in self.runq_depends[revdep]:
+                    if dep in unchecked:
+                        alldeps = 0
+                if alldeps == 1:
+                    if revdep in unchecked:
+                        buildable.append(revdep)
+
+        for task in range(len(self.runq_fnid)):
+            if task not in unchecked:
+                continue
+            fn = self.taskData.fn_index[self.runq_fnid[task]]
+            taskname = self.runq_task[task]
+            stampfile = "%s.%s" % (self.dataCache.stamp[fn], taskname)
+            # If the stamp is missing its not current
+            if not os.access(stampfile, os.F_OK):
+                del unchecked[task]
+                notcurrent.append(task)
+                check_buildable(self, task, buildable)
+                continue
+            # If its a 'nostamp' task, it's not current
+            taskdep = self.dataCache.task_deps[fn]
+            if 'nostamp' in taskdep and task in taskdep['nostamp']:
+                del unchecked[task]
+                notcurrent.append(task)
+                check_buildable(self, task, buildable)
+                continue
+
+        while (len(buildable) > 0):
+            nextbuildable = []
+            for task in buildable:
+                if task in unchecked:
+                    fn = self.taskData.fn_index[self.runq_fnid[task]]
+                    taskname = self.runq_task[task]
+                    stampfile = "%s.%s" % (self.dataCache.stamp[fn], taskname)
+                    iscurrent = True
+
+                    t1 = os.stat(stampfile)[stat.ST_MTIME]
+                    for dep in self.runq_depends[task]:
+                        if iscurrent:
+                            fn2 = self.taskData.fn_index[self.runq_fnid[dep]]
+                            taskname2 = self.runq_task[dep]
+                            stampfile2 = "%s.%s" % (self.dataCache.stamp[fn2], taskname2)
+                            if fn == fn2 or (fulldeptree and fn2 not in stampwhitelist):
+                                if dep in notcurrent:
+                                    iscurrent = False
+                                else:
+                                    t2 = os.stat(stampfile2)[stat.ST_MTIME]
+                                    if t1 < t2:
+                                        iscurrent = False
+                    del unchecked[task]
+                    if iscurrent:
+                        current.append(task)
+                    else:
+                        notcurrent.append(task)
+
+                check_buildable(self, task, nextbuildable)
+
+            buildable = nextbuildable
+
+        #for task in range(len(self.runq_fnid)):
+        #    fn = self.taskData.fn_index[self.runq_fnid[task]]
+        #    taskname = self.runq_task[task]
+        #    print "%s %s.%s" % (task, taskname, fn)
+
+        #print "Unchecked: %s" % unchecked
+        #print "Current: %s" % current
+        #print "Not current: %s" % notcurrent
+
+        if len(unchecked) > 0:
+            bb.fatal("check_stamps fatal internal error")
+        return current
+
+    def check_stamp_task(self, task):
+
+        if self.stamppolicy == "perfile":
+            fulldeptree = False
+        else:
+            fulldeptree = True
+            stampwhitelist = []
+            if self.stamppolicy == "whitelist":
+                stampwhitelist = self.stampfnwhitelist
+
+        fn = self.taskData.fn_index[self.runq_fnid[task]]
+        taskname = self.runq_task[task]
+        stampfile = "%s.%s" % (self.dataCache.stamp[fn], taskname)
+        # If the stamp is missing its not current
+        if not os.access(stampfile, os.F_OK):
+            bb.msg.debug(2, bb.msg.domain.RunQueue, "Stampfile %s not available\n" % stampfile)
+            return False
+        # If its a 'nostamp' task, it's not current
+        taskdep = self.dataCache.task_deps[fn]
+        if 'nostamp' in taskdep and taskname in taskdep['nostamp']:
+            bb.msg.debug(2, bb.msg.domain.RunQueue, "%s.%s is nostamp\n" % (fn, taskname))
+            return False
+
+        iscurrent = True
+        t1 =  os.stat(stampfile)[stat.ST_MTIME]
+        for dep in self.runq_depends[task]:
+            if iscurrent:
+                fn2 = self.taskData.fn_index[self.runq_fnid[dep]]
+                taskname2 = self.runq_task[dep]
+                stampfile2 = "%s.%s" % (self.dataCache.stamp[fn2], taskname2)
+                if fn == fn2 or (fulldeptree and fn2 not in stampwhitelist):
+                    try:
+                        t2 = os.stat(stampfile2)[stat.ST_MTIME]
+                        if t1 < t2:
+                            bb.msg.debug(2, bb.msg.domain.RunQueue, "Stampfile %s < %s" % (stampfile,stampfile2))
+                            iscurrent = False
+                    except:
+                        bb.msg.debug(2, bb.msg.domain.RunQueue, "Exception reading %s for %s" % (stampfile2 ,stampfile))
+                        iscurrent = False
+
+        return iscurrent
+
     def execute_runqueue(self):
         """
         Run the tasks in a queue prepared by prepare_runqueue
@@ -653,7 +845,7 @@ class RunQueue:
                     failed_fnids = self.finish_runqueue()
             if len(failed_fnids) == 0:
                 return failures
-            if self.taskData.abort:
+            if not self.taskData.tryaltconfigs:
                 raise bb.runqueue.TaskFailure(failed_fnids)
             for fnid in failed_fnids:
                 #print "Failure: %s %s %s" % (fnid, self.taskData.fn_index[fnid],  self.runq_task[fnid])
@@ -721,18 +913,7 @@ class RunQueue:
         def sigint_handler(signum, frame):
             raise KeyboardInterrupt
 
-        # RP - this code allows tasks to run out of the correct order - disabled, FIXME
-        # Find any tasks with current stamps and remove them from the queue
-        #for task1 in range(len(self.runq_fnid)):
-        #    task = self.prio_map[task1]
-        #    fn = self.taskData.fn_index[self.runq_fnid[task]]
-        #    taskname = self.runq_task[task]
-        #    if bb.build.stamp_is_current(taskname, self.dataCache, fn):
-        #        bb.msg.debug(2, bb.msg.domain.RunQueue, "Stamp current task %s (%s)" % (task, self.get_user_idstring(task)))
-        #        self.runq_running[task] = 1
-        #        self.task_complete(task)
-        #        self.stats.taskCompleted()
-        #        self.stats.taskSkipped()
+        event.fire(bb.event.StampUpdate(self.target_pairs, self.dataCache.stamp, self.cfgdata))
 
         while True:
             task = self.sched.next()
@@ -740,7 +921,7 @@ class RunQueue:
                 fn = self.taskData.fn_index[self.runq_fnid[task]]
 
                 taskname = self.runq_task[task]
-                if bb.build.stamp_is_current(taskname, self.dataCache, fn):
+                if self.check_stamp_task(task):
                     bb.msg.debug(2, bb.msg.domain.RunQueue, "Stamp current task %s (%s)" % (task, self.get_user_idstring(task)))
                     self.runq_running[task] = 1
                     self.task_complete(task)
@@ -749,6 +930,8 @@ class RunQueue:
                     continue
 
                 bb.msg.note(1, bb.msg.domain.RunQueue, "Running task %d of %d (ID: %s, %s)" % (self.stats.completed + self.active_builds + 1, len(self.runq_fnid), task, self.get_user_idstring(task)))
+                sys.stdout.flush()
+                sys.stderr.flush()
                 try: 
                     pid = os.fork() 
                 except OSError, e: 
@@ -763,8 +946,9 @@ class RunQueue:
                     newsi = os.open('/dev/null', os.O_RDWR)
                     os.dup2(newsi, sys.stdin.fileno())
                     self.cooker.configuration.cmd = taskname[3:]
-                    try: 
-                        self.cooker.tryBuild(fn, False)
+                    bb.data.setVar("__RUNQUEUE_DO_NOT_USE_EXTERNALLY", self, self.cooker.configuration.data)
+                    try:
+                        self.cooker.tryBuild(fn)
                     except bb.build.EventException:
                         bb.msg.error(bb.msg.domain.Build, "Build of " + fn + " " + taskname + " failed")
                         sys.exit(1)
@@ -786,6 +970,8 @@ class RunQueue:
                     bb.msg.error(bb.msg.domain.RunQueue, "Task %s (%s) failed" % (task, self.get_user_idstring(task)))
                     self.failed_fnids.append(self.runq_fnid[task])
                     self.stats.taskFailed()
+                    if not self.taskData.abort:
+                        continue
                     break
                 self.task_complete(task)
                 self.stats.taskCompleted()
@@ -856,3 +1042,13 @@ class RunQueue:
                         self.runq_weight[task], 
                         self.runq_depends[task], 
                         self.runq_revdeps[task]))
+
+
+def check_stamp_fn(fn, taskname, d):
+    rq = bb.data.getVar("__RUNQUEUE_DO_NOT_USE_EXTERNALLY", d)
+    fnid = rq.taskData.getfn_id(fn)
+    taskid = rq.get_task_id(fnid, taskname)
+    if taskid is not None:
+        return rq.check_stamp_task(taskid)
+    return None
+
